structure Zeit = 
  struct
     type zeit = int
     val zeitpunkte = (* Zeitpunkte [0,1,...,49] *)
         List.tabulate (50, fn x => x)  

     structure Q = Quantoren(struct type obj = zeit 
                                    val objects = zeitpunkte 
                             end)

     (* Temporaloperatoren als Quantoren ueber Zeitpunkte *)

     fun F S t = Q.exists (fn s => s > t) S   (* wird einmal *)
     fun G S t = Q.forall (fn s => s > t) S   (* wird immer *)

     fun H S t = Q.exists (fn s => s < t) S   (* war einmal *)
     fun P S t = Q.forall (fn s => s < t) S   (* war immer *)

     fun Since (S1,S2) t = 
         case (List.find (fn s => s < t andalso S1 s) zeitpunkte) 
          of SOME s => List.all (fn r => not(s < r andalso r <= t) 
                                         orelse S2 r) zeitpunkte
           | NONE => false

     fun Until (S1,S2) t = 
         case (List.find (fn s => t < s andalso S1 s) zeitpunkte) 
          of SOME s => List.all (fn r => not(t <= r andalso r < s)
                                         orelse S2 r) zeitpunkte
           | NONE => false

     (* Werte von Zeitadverbien *)

     fun gestern t = t-1
     fun heute t   = t
     fun morgen t  = t+1

     fun immer S t      = List.all S zeitpunkte
     fun irgendwann S t = List.exists S zeitpunkte
     fun nie S t        = not (irgendwann S t)

     (* Werte von temporalen Junktoren *)

     val seit = Since 
     val bis = Until
  end

